//Feedback
#include <Wire.h>
#include <Servo.h>
//Bluetooth
#include <SoftwareSerial.h>
#include <Time.h>

//Switch
int flipSwitch=2;
//Feedback
int minDegrees=0;
int maxDegrees=180;
int minFeedback=106.90625;
int maxFeedback=448.21875;
int baseServo=0;
int shoulderServo=1;
int elbowServo=2;
int wristServo=3;
//Bluetooth
int gPowerPin = 8;
int gKeyPin = 9;
int gRxPin = 10;
int gTxPin = 11;
int gIsMaster = 1;
SoftwareSerial BTSerial(gRxPin, gTxPin);
//Data Arrays
byte baseData[3]={254,0,0};
byte shoulderData[3]={254,1,0};
byte elbowData[3]={254,2,0};
byte wristData[3]={254,3,0};
byte switchData[3]={254,4,0};

//Bluetooth
void BtReset(){
  Serial.println("BtReset");    
  BTSerial.flush();
  delay(500);
  digitalWrite(gPowerPin, LOW);  
  delay(2000); 
  digitalWrite(gPowerPin, HIGH);
  delay(500);
}

void BtSetCmdMode(int i_cmdMode = 1){
  Serial.print("BtSetCmdMode ");    
  Serial.println(i_cmdMode);    
  digitalWrite(gKeyPin, (1 == i_cmdMode)?HIGH:LOW);
  BtReset();
  BTSerial.begin((1 == i_cmdMode)?38400:57600);//57600
}

void BtSetupRole(){
  BTSerial.begin(38400);
  BtSetCmdMode(1);
  BTSerial.println("AT");
  delay(1000);
  BTSerial.println("AT+UART=57600,0,0");
  delay(1000);
  BTSerial.println("AT+ROLE=1");
  Serial.println("Set as Master");
  delay(1000); 
  BtSetCmdMode(0); 
}

void BtStartMaster(){
  BtSetCmdMode(1);  
  BTSerial.println("AT+INIT");
  Serial.println("AT command: AT+INIT");
  delay(500); 
  BTSerial.println("AT+LINK=13,12,235271");
  Serial.println("AT command: AT+LINK=");
  delay(100); 
  BtSetCmdMode(0);
}

//Feedback
void calibrate(Servo servo, int analogPin, int minPos, int maxPos)
{
  // Move to the minimum position and record the feedback value
  servo.write(minPos);
  minDegrees = minPos;
  delay(2000); // make sure it has time to get there and settle
  minFeedback = analogRead(analogPin);
  Serial.println(minFeedback);
  
  // Move to the maximum position and record the feedback value
  servo.write(maxPos);
  maxDegrees = maxPos;
  delay(2000); // make sure it has time to get there and settle
  maxFeedback = analogRead(analogPin);
  Serial.println(maxFeedback);
}

int getPos(int servo)
{
  int angle=map(analogRead(servo), minFeedback, maxFeedback, maxDegrees, minDegrees);
  if (angle<0){
    return 0;
  }else if(angle>180){
    return 180;
  }else{
    return angle;
  }
}
int getPosMini()
{
  int angle=map(analogRead(3), 162, 466.5, 0, 180);
  if (angle<0){
    return 0;
  }else if(angle>180){
    return 180;
  }else{
    return angle;
  }
}

void setup()
{
  Serial.begin(9600);
  pinMode(flipSwitch, INPUT);
  pinMode(gPowerPin, OUTPUT);
  digitalWrite(gPowerPin, LOW);
  pinMode(gKeyPin, OUTPUT);
  digitalWrite(gKeyPin, LOW);
  BtSetupRole();
  BtStartMaster();
  BTSerial.flush();
}

void loop(){
  int pos=getPos(baseServo);
  if (baseData[2]>pos+1||baseData[2]<pos-1){
    baseData[2]=pos;
    BTSerial.write(baseData,3);
    Serial.println("\n");
    Serial.println("Base");
    Serial.println(pos);
  }
  pos=getPos(shoulderServo);
  if (shoulderData[2]>pos+1||shoulderData[2]<pos-1){
    shoulderData[2]=pos;
    BTSerial.write(shoulderData,3);
    Serial.println("\n");
    Serial.println("Shoulder");
    Serial.println(pos);
  }
  pos=180-getPos(elbowServo);
  if (elbowData[2]>pos+1||elbowData[2]<pos-1){
    elbowData[2]=pos;
    BTSerial.write(elbowData,3);
    Serial.println("\n");
    Serial.println("Elbow");
    Serial.println(pos);
  }
  pos=getPosMini();
  if (wristData[2]>pos+1||wristData[2]<pos-1){
    wristData[2]=pos;
    BTSerial.write(wristData,3);
    Serial.println("\n");
    Serial.println("Wrist");
    Serial.println(pos);
  }
  pos=digitalRead(flipSwitch);
  if (switchData[2]!=pos){
    switchData[2]=pos;
    BTSerial.write(switchData,3);
    Serial.println("\n");
    Serial.println("Switch");
    Serial.println(pos);
  }
  delay(100);
}
