// include needed librarys
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>
#include <SoftwareSerial.h>
#include <Time.h>
 
int gPowerPin = 8;
int gKeyPin = 9;
int gRxPin = 10;
int gTxPin = 11;
SoftwareSerial BTSerial(gRxPin, gTxPin);

void BtReset(void){
  Serial.println("BtReset");    
  BTSerial.flush();
  delay(500);  
  //Method 1: Using the power
  digitalWrite(gPowerPin, LOW);  
  delay(2000); 
  digitalWrite(gPowerPin, HIGH);
  delay(500);
}

void BtSetCmdMode(int i_cmdMode = 1){
  Serial.print("BtSetCmdMode ");    
  Serial.println(i_cmdMode);    
  digitalWrite(gKeyPin, (1 == i_cmdMode)?HIGH:LOW);
  BtReset();
  BTSerial.begin((1 == i_cmdMode)?38400:57600);//57600
}

void BtSetupRole(){
  BTSerial.begin(38400);
  BtSetCmdMode(1);
  BTSerial.println("AT");
  delay(1000);
  BTSerial.println("AT+UART=57600,0,0");
  delay(1000);
  BTSerial.println("AT+ROLE=0");
  Serial.println("Set as Slave");
  delay(1000); 
  BtSetCmdMode(0); 
}

// create servo driver (defaults to addr 40)
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

void setup()
{
  pinMode(3,OUTPUT);
  pinMode(gPowerPin, OUTPUT);
  digitalWrite(gPowerPin, LOW);
  pinMode(gKeyPin, OUTPUT);
  digitalWrite(gKeyPin, LOW);
  Serial.begin(9600);
  BtSetupRole();
  pwm.begin();
  pwm.setPWMFreq(50);
  BTSerial.flush();
}
 
// helper function that uses angles rather than pulse ticks
void setAngle( int channel, int angle )
{
  // 4096 ticks is 20,000 us
  // Angle 0 is 1,000 us
  // angle 180 is 2,000 us
  long ticks = ((500L + (2000L*angle/180L))*4096L)/20000L;
  // update the servo channel with the new pusle
  pwm.setPWM(channel, 0, ticks);
}
 
// helper function to execute a frame list;
void execute( int *cmd )
{
  while( *cmd >= 0 )
  {
    setAngle( *cmd++, *cmd++);
  }
}

int safeRead(){
  while(!BTSerial.available()){}
  Serial.println("Waiting");
  return BTSerial.read();
}

// main loop
void loop()
{
  int tmpArray[2]={0,0};
  if(254==safeRead()){
    Serial.print("\n");
    tmpArray[0]=safeRead();
    tmpArray[1]=safeRead();
    Serial.println(tmpArray[0]);
    Serial.println(tmpArray[1]);
    if (tmpArray[0]<4){
      setAngle(tmpArray[0],tmpArray[1]);
    }else{
      digitalWrite(3,tmpArray[1]);
    }
  }
}
